@@ -164,13 +164,13 @@ client_handle_set_configuration(struct lldpd *cfg, enum hmsg_type *type,
 	}
 	if (CHANGED(c_cap_advertise)) {
 		log_debug("rpc", "%s chassis capabilities advertisement",
-		    config->c_promisc?"enable":"disable");
+		    config->c_cap_advertise?"enable":"disable");
 		cfg->g_config.c_cap_advertise = config->c_cap_advertise;
 		levent_update_now(cfg);
 	}
 	if (CHANGED(c_mgmt_advertise)) {
 		log_debug("rpc", "%s management addresses advertisement",
-		    config->c_promisc?"enable":"disable");
+		    config->c_mgmt_advertise?"enable":"disable");
 		cfg->g_config.c_mgmt_advertise = config->c_mgmt_advertise;
 		levent_update_now(cfg);
 	}
@@ -246,7 +246,7 @@ static ssize_t
 client_handle_get_local_chassis(struct lldpd *cfg, enum hmsg_type *type,
     void *input, int input_len, void **output, int *subscribed)
 {
-    struct lldpd_chassis *chassis = LOCAL_CHASSIS(cfg);
+	struct lldpd_chassis *chassis = LOCAL_CHASSIS(cfg);
 	ssize_t output_len;
 
 	log_debug("rpc", "client request the local chassis");
@@ -297,6 +297,139 @@ client_handle_get_interface(struct lldpd *cfg, enum hmsg_type *type,
 	return 0;
 }
 
+/* Return all available information related to an interface
+   Input:  name of the interface (serialized)
+   Output: Information about the interface (lldpd_hardware)
+*/
+static ssize_t
+client_handle_get_default_port(struct lldpd *cfg, enum hmsg_type *type,
+    void *input, int input_len, void **output, int *subscribed)
+{
+	log_debug("rpc", "client request the default local port");
+	ssize_t output_len = lldpd_port_serialize(cfg->g_default_local_port, output);
+	if (output_len <= 0) {
+		*type = NONE;
+		return 0;
+	}
+	return output_len;
+}
+
+static int
+_client_handle_set_port(struct lldpd *cfg,
+    struct lldpd_port *port, struct lldpd_port_set *set)
+{
+#ifdef ENABLE_LLDPMED
+	struct lldpd_med_loc *loc = NULL;
+#endif
+	if (set->local_id) {
+		log_debug("rpc", "requested change to Port ID");
+		free(port->p_id);
+		port->p_id = strdup(set->local_id);
+		port->p_id_len = strlen(set->local_id);
+		port->p_id_subtype = LLDP_PORTID_SUBTYPE_LOCAL;
+	}
+	if (set->local_descr) {
+		log_debug("rpc", "requested change to Port Description");
+		free(port->p_descr);
+		port->p_descr = strdup(set->local_descr);
+	}
+	switch (set->rxtx) {
+	case LLDPD_RXTX_TXONLY:
+		log_debug("rpc", "requested TX only mode");
+		port->p_disable_rx = 1;
+		port->p_disable_tx = 0;
+		break;
+	case LLDPD_RXTX_RXONLY:
+		log_debug("rpc", "requested RX only mode");
+		port->p_disable_rx = 0;
+		port->p_disable_tx = 1;
+		break;
+	case LLDPD_RXTX_BOTH:
+		log_debug("rpc", "requested RX/TX mode");
+		port->p_disable_rx = port->p_disable_tx = 0;
+		break;
+	case LLDPD_RXTX_DISABLED:
+		log_debug("rpc", "requested disabled mode");
+		port->p_disable_rx = port->p_disable_tx = 1;
+		break;
+	}
+#ifdef ENABLE_LLDPMED
+	if (set->med_policy && set->med_policy->type > 0) {
+		log_debug("rpc", "requested change to MED policy");
+		if (set->med_policy->type > LLDP_MED_APPTYPE_LAST) {
+			log_warnx("rpc", "invalid policy provided: %d",
+			    set->med_policy->type);
+			return -1;
+		}
+		memcpy(&port->p_med_policy[set->med_policy->type - 1],
+		    set->med_policy, sizeof(struct lldpd_med_policy));
+		port->p_med_cap_enabled |= LLDP_MED_CAP_POLICY;
+	}
+	if (set->med_location && set->med_location->format > 0) {
+		char *newdata = NULL;
+		log_debug("rpc", "requested change to MED location");
+		if (set->med_location->format > LLDP_MED_LOCFORMAT_LAST) {
+			log_warnx("rpc", "invalid location format provided: %d",
+			    set->med_location->format);
+			return -1;
+		}
+		loc = \
+		    &port->p_med_location[set->med_location->format - 1];
+		free(loc->data);
+		memcpy(loc, set->med_location, sizeof(struct lldpd_med_loc));
+		if (!loc->data || !(newdata = malloc(loc->data_len))) loc->data_len = 0;
+		if (newdata) memcpy(newdata, loc->data, loc->data_len);
+		loc->data = newdata;
+		port->p_med_cap_enabled |= LLDP_MED_CAP_LOCATION;
+	}
+	if (set->med_power) {
+		log_debug("rpc", "requested change to MED power");
+		memcpy(&port->p_med_power, set->med_power,
+		    sizeof(struct lldpd_med_power));
+		switch (set->med_power->devicetype) {
+		case LLDP_MED_POW_TYPE_PD:
+			port->p_med_cap_enabled |= LLDP_MED_CAP_MDI_PD;
+			port->p_med_cap_enabled &= ~LLDP_MED_CAP_MDI_PSE;
+			break;
+		case LLDP_MED_POW_TYPE_PSE:
+			port->p_med_cap_enabled |= LLDP_MED_CAP_MDI_PSE;
+			port->p_med_cap_enabled &= ~LLDP_MED_CAP_MDI_PD;
+			break;
+		}
+	}
+#endif
+#ifdef ENABLE_DOT3
+	if (set->dot3_power) {
+		log_debug("rpc", "requested change to Dot3 power");
+		memcpy(&port->p_power, set->dot3_power,
+		    sizeof(struct lldpd_dot3_power));
+	}
+#endif
+#ifdef ENABLE_CUSTOM
+	if (set->custom_list_clear) {
+		log_debug("rpc", "requested custom TLVs clear");
+		lldpd_custom_list_cleanup(port);
+	} else {
+		if (set->custom) {
+			struct lldpd_custom *custom;
+			log_debug("rpc", "requested custom TLV add");
+			if ((custom = malloc(sizeof(struct lldpd_custom)))) {
+				memcpy(custom, set->custom, sizeof(struct lldpd_custom));
+				if ((custom->oui_info = malloc(custom->oui_info_len))) {
+					memcpy(custom->oui_info, set->custom->oui_info, custom->oui_info_len);
+					TAILQ_INSERT_TAIL(&port->p_custom_list, custom, next);
+				} else {
+					free(custom);
+					log_warn("rpc", "could not allocate memory for custom TLV info");
+				}
+			} else
+				log_warn("rpc", "could not allocate memory for custom TLV");
+		}
+	}
+#endif
+	return 0;
+}
+
 /* Set some port related settings (policy, location, power)
    Input: name of the interface, policy/location/power setting to be modified
    Output: nothing
@@ -308,9 +441,6 @@ client_handle_set_port(struct lldpd *cfg, enum hmsg_type *type,
 	int ret = 0;
 	struct lldpd_port_set *set = NULL;
 	struct lldpd_hardware *hardware = NULL;
-#ifdef ENABLE_LLDPMED
-	struct lldpd_med_loc *loc = NULL;
-#endif
 
 	if (lldpd_port_set_unserialize(input, input_len, &set) <= 0) {
 		*type = NONE;
@@ -322,99 +452,23 @@ client_handle_set_port(struct lldpd *cfg, enum hmsg_type *type,
 	}
 
 	/* Search the appropriate hardware */
-	log_debug("rpc", "client request change to port %s", set->ifname);
-	TAILQ_FOREACH(hardware, &cfg->g_hardware, h_entries)
-	    if (!strcmp(hardware->h_ifname, set->ifname)) {
-		    struct lldpd_port *port = &hardware->h_lport;
-		    if (set->local_id) {
-			    log_debug("rpc", "requested change to Port ID");
-			    free(port->p_id);
-			    port->p_id = strdup(set->local_id);
-			    port->p_id_len = strlen(set->local_id);
-			    port->p_id_subtype = LLDP_PORTID_SUBTYPE_LOCAL;
-		    }
-		    if (set->local_descr) {
-			    log_debug("rpc", "requested change to Port Description");
-			    free(port->p_descr);
-			    port->p_descr = strdup(set->local_descr);
-		    }
-#ifdef ENABLE_LLDPMED
-		    if (set->med_policy && set->med_policy->type > 0) {
-			    log_debug("rpc", "requested change to MED policy");
-			    if (set->med_policy->type > LLDP_MED_APPTYPE_LAST) {
-				    log_warnx("rpc", "invalid policy provided: %d",
-					set->med_policy->type);
+	if (strlen(set->ifname) == 0) {
+		log_debug("rpc", "client request change to default port");
+		if (_client_handle_set_port(cfg, cfg->g_default_local_port, set) == -1)
+			goto set_port_finished;
+		ret = 1;
+	} else {
+		log_debug("rpc", "client request change to port %s", set->ifname);
+		TAILQ_FOREACH(hardware, &cfg->g_hardware, h_entries) {
+		    if (!strcmp(hardware->h_ifname, set->ifname)) {
+			    struct lldpd_port *port = &hardware->h_lport;
+			    if (_client_handle_set_port(cfg, port, set) == -1)
 				    goto set_port_finished;
-			    }
-			    memcpy(&port->p_med_policy[set->med_policy->type - 1],
-				set->med_policy, sizeof(struct lldpd_med_policy));
-			    port->p_med_cap_enabled |= LLDP_MED_CAP_POLICY;
+			    ret = 1;
+			    break;
 		    }
-		    if (set->med_location && set->med_location->format > 0) {
-			    char *newdata = NULL;
-			    log_debug("rpc", "requested change to MED location");
-			    if (set->med_location->format > LLDP_MED_LOCFORMAT_LAST) {
-				    log_warnx("rpc", "invalid location format provided: %d",
-					set->med_location->format);
-				    goto set_port_finished;
-			    }
-			    loc = \
-				&port->p_med_location[set->med_location->format - 1];
-			    free(loc->data);
-			    memcpy(loc, set->med_location, sizeof(struct lldpd_med_loc));
-			    if (!loc->data || !(newdata = malloc(loc->data_len))) loc->data_len = 0;
-			    if (newdata) memcpy(newdata, loc->data, loc->data_len);
-			    loc->data = newdata;
-			    port->p_med_cap_enabled |= LLDP_MED_CAP_LOCATION;
-		    }
-		    if (set->med_power) {
-			    log_debug("rpc", "requested change to MED power");
-			    memcpy(&port->p_med_power, set->med_power,
-				sizeof(struct lldpd_med_power));
-			    switch (set->med_power->devicetype) {
-			    case LLDP_MED_POW_TYPE_PD:
-				    port->p_med_cap_enabled |= LLDP_MED_CAP_MDI_PD;
-				    port->p_med_cap_enabled &= ~LLDP_MED_CAP_MDI_PSE;
-				    break;
-			    case LLDP_MED_POW_TYPE_PSE:
-				    port->p_med_cap_enabled |= LLDP_MED_CAP_MDI_PSE;
-				    port->p_med_cap_enabled &= ~LLDP_MED_CAP_MDI_PD;
-				    break;
-			    }
-		    }
-#endif
-#ifdef ENABLE_DOT3
-		    if (set->dot3_power) {
-			    log_debug("rpc", "requested change to Dot3 power");
-			    memcpy(&port->p_power, set->dot3_power,
-				sizeof(struct lldpd_dot3_power));
-		    }
-#endif
-#ifdef ENABLE_CUSTOM
-		    if (set->custom_list_clear) {
-			    log_debug("rpc", "requested custom TLVs clear");
-			    lldpd_custom_list_cleanup(port);
-		    } else 
-		    if (set->custom) {
-			    struct lldpd_custom *custom;
-			    log_debug("rpc", "requested custom TLV add");
-			    if ((custom = malloc(sizeof(struct lldpd_custom)))) {
-				    memcpy(custom, set->custom, sizeof(struct lldpd_custom));
-				    if ((custom->oui_info = malloc(custom->oui_info_len))) {
-					    memcpy(custom->oui_info, set->custom->oui_info, custom->oui_info_len);
-					    TAILQ_INSERT_TAIL(&port->p_custom_list, custom, next);
-				    } else {
-					    free(custom);
-					    log_warn("rpc", "could not allocate memory for custom TLV info");
-				    }
-			    } else
-				    log_warn("rpc", "could not allocate memory for custom TLV");
-		    }
-#endif
-
-		    ret = 1;
-		    break;
-	    }
+		}
+	}
 
 	if (ret == 0)
 		log_warn("rpc", "no interface %s found", set->ifname);
@@ -467,6 +521,7 @@ static struct client_handle client_handles[] = {
 	{ SET_CONFIG,		"Set configuration", client_handle_set_configuration },
 	{ GET_INTERFACES,	"Get interfaces",    client_handle_get_interfaces },
 	{ GET_INTERFACE,	"Get interface",     client_handle_get_interface },
+	{ GET_DEFAULT_PORT,	"Get default port",  client_handle_get_default_port },
 	{ GET_CHASSIS,		"Get local chassis", client_handle_get_local_chassis },
 	{ SET_PORT,		"Set port",          client_handle_set_port },
 	{ SUBSCRIBE,		"Subscribe",         client_handle_subscribe },
